from tira.local_client import Client

import pandas as pd
import unittest

# .. todo:: this file still uses "unclean" assertions and should be converted to use unittest assertions

def retrieval_submission(queries):
    queries = pd.DataFrame([{'qid': str(i)} for i in queries])
    tira = Client('tests/resources/')
    retriever =  tira.pt.from_retriever_submission('ir-benchmarks/tira-ir-starters/retriever', dataset='d1')
    
    ret = retriever(queries)
    return [{'qid': i['qid'], 'docno': i['docno'], 'score': i['score']} for _, i in ret.iterrows()]

class TestPtFromRetrieverSubmissionLoaderTest(unittest.TestCase):
    def test_for_multiple_queries(self):
        expected = [
            {'qid': '1', 'docno': 'doc-1', 'score':10},
            {'qid': '1', 'docno': 'doc-2', 'score':9},
            {'qid': '3', 'docno': 'doc-3', 'score': 1},
        ]
        
        actual = retrieval_submission([1, 3])

        assert len(actual) == 3
        assert actual[0] == expected[0]
        assert actual[1] == expected[1]
        assert actual[2] == expected[2]

    def test_for_single_query_no_1(self):
        expected = [
            {'qid': '1', 'docno': 'doc-1', 'score':10},
            {'qid': '1', 'docno': 'doc-2', 'score':9},
        ]
        
        actual = retrieval_submission([1])

        assert len(actual) == 2
        assert actual[0] == expected[0]
        assert actual[1] == expected[1]

    def test_for_single_query_no_3(self):
        expected = [
            {'qid': '3', 'docno': 'doc-3', 'score': 1},
        ]
        
        actual = retrieval_submission([3])

        assert len(actual) == 1
        assert actual[0] == expected[0]

    def test_retrieval_submission_from_rest_api(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.from_submission('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', pt.get_dataset("irds:disks45/nocr/trec-robust-2004"))
        assert len(q(pd.DataFrame([{'qid': '306'}]))) == 1000
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['docno'] == 'LA021790-0114'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['qid'] == '306'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['docno'] == 'FBIS4-47956'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['qid'] == '306'

    def test_retrieval_submission_from_rest_api_different_id(self):
        from tira.rest_api_client import Client
        from tira.third_party_integrations import ensure_pyterrier_is_loaded
        import pyterrier as pt
        ensure_pyterrier_is_loaded()
        tira = Client()

        q = tira.pt.from_submission('ir-benchmarks/tira-ir-starter/BM25 Re-Rank (tira-ir-starter-pyterrier)', pt.get_dataset("irds:ir-benchmarks/disks45-nocr-trec-robust-2004-20230209-training"))
        assert len(q(pd.DataFrame([{'qid': '306'}]))) == 1000
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['docno'] == 'LA021790-0114'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[0].to_dict()['qid'] == '306'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['docno'] == 'FBIS4-47956'
        assert q(pd.DataFrame([{'qid': '306'}])).iloc[999].to_dict()['qid'] == '306'


#!/usr/bin/env python
import argparse
from tira.rest_api_client import Client as RestClient
import os
import shutil
import logging

from typing import TYPE_CHECKING
if TYPE_CHECKING:
    from .tira_client import TiraClient


def parse_args():
    parser = argparse.ArgumentParser(prog='tira-cli')
    
    parser.add_argument('-v', '--verbose', required=False, default=False, action='store_true')
    subparsers = parser.add_subparsers(dest='command')
    parser_download = subparsers.add_parser('download', help='Download runs or datasets from TIRA.io')
    parser_download.add_argument('--approach', required=False, default=None, help='Download the outputs of the specified approach. Usage: --approach <task-id>/<user-id>/<approach-name>')
    parser_download.add_argument('--dataset', required=True, help='The dataset.')
    
    parser_upload = subparsers.add_parser('upload', help='Upload runs or datasets to TIRA.io')

    parser_login = subparsers.add_parser('login', help='Login your TIRA client to the tira server.')
    parser_login.add_argument('--token', required=True, default=None, help='The token to login to the server.')

    args = parser.parse_args()

    return args


def download_run(client, approach, dataset):
    help()

def main():
    args = parse_args()
    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)

    client: "TiraClient" = RestClient()

    if args.command == 'download':
        if args.approach is not None:
            print(client.get_run_output(args.approach, args.dataset))
        else:
            print(client.download_dataset(None, args.dataset))
    if args.command == 'login':
        client.login(args.token)



from tira.third_party_integrations import persist_and_normalize_run
import pandas as pd
import unittest

# .. todo:: there are no assertions in this file

class TestPersistingRunFiles(unittest.TestCase):

    def test_variant_01(self):
        run = pd.DataFrame([{"qid": "1", "score": "1", "docno": "d1"},  {"qid": "1", "score": "0", "docno": "d2"},  {"qid": "1", "score": "2", "docno": "d3"}])
        persist_and_normalize_run(run, 'system_name', default_output='/tmp/variant_01.run.txt')
        
    def test_variant_02(self):
        run = pd.DataFrame([{"qid": "1", "score": "1", "docno": "d1"},  {"qid": "1", "score": "0", "docno": "d2"},  {"qid": "1", "score": "2", "docno": "d3"}])
        persist_and_normalize_run(run, 'system_name', output_file='/tmp/variant_02.run.txt')

    def test_variant_03(self):
        run = pd.DataFrame([{"qid": "1", "score": "1", "docno": "d1"},  {"qid": "1", "score": "0", "docno": "d2"},  {"qid": "1", "score": "2", "docno": "d3"}])
        persist_and_normalize_run(run, 'system_name')



IRDS_TO_TIREX_DATASET = {
    'clueweb09/en/trec-web-2009': 'clueweb09-en-trec-web-2009-20230107-training',
    'clueweb09/en/trec-web-2010': 'clueweb09-en-trec-web-2010-20230107-training',
    'clueweb09/en/trec-web-2011': 'clueweb09-en-trec-web-2011-20230107-training',
    'clueweb09/en/trec-web-2012': 'clueweb09-en-trec-web-2012-20230107-training',
    'clueweb12/trec-web-2013': 'clueweb12-trec-web-2013-20230107-training',
    'clueweb12/trec-web-2014': 'clueweb12-trec-web-2014-20230107-training',
    'disks45/nocr/trec-robust-2004': 'disks45-nocr-trec-robust-2004-20230209-training',
    'antique/test': 'antique-test-20230107-training',
    'argsme/2020-04-01/touche-2020-task-1': 'argsme-touche-2020-task-1-20230209-training',
    'argsme/2020-04-01/touche-2021-task-1': 'argsme-touche-2021-task-1-20230209-training',
    'clueweb12/touche-2020-task-2': 'clueweb12-touche-2020-task-2-20230209-training',
    'clueweb12/touche-2021-task-2': 'clueweb12-touche-2021-task-2-20230209-training',
    'clueweb12/b13/trec-misinfo-2019': 'clueweb12-trec-misinfo-2019-20240214-training',
    'cord19/fulltext/trec-covid': 'cord19-fulltext-trec-covid-20230107-training',
    'cranfield': 'cranfield-20230107-training',
    'disks45/nocr/trec7': 'disks45-nocr-trec7-20230209-training',
    'disks45/nocr/trec8': 'disks45-nocr-trec8-20230209-training',
    'gov/trec-web-2002': 'gov-trec-web-2002-20230209-training',
    'gov/trec-web-2003': 'gov-trec-web-2003-20230209-training',
    'gov/trec-web-2004': 'gov-trec-web-2004-20230209-training',
    'gov2/trec-tb-2004': 'gov2-trec-tb-2004-20230209-training',
    'gov2/trec-tb-2005': 'gov2-trec-tb-2005-20230209-training',
    'gov2/trec-tb-2006': 'gov2-trec-tb-2006-20230209-training',
    'medline/2004/trec-genomics-2004': 'medline-2004-trec-genomics-2004-20230107-training',
    'medline/2004/trec-genomics-2005': 'medline-2004-trec-genomics-2005-20230107-training',
    'medline/2017/trec-pm-2017': 'medline-2017-trec-pm-2017-20230211-training',
    'medline/2017/trec-pm-2018': 'medline-2017-trec-pm-2018-20230211-training',
    'msmarco-passage/trec-dl-2019/judged': 'msmarco-passage-trec-dl-2019-judged-20230107-training',
    'msmarco-passage/trec-dl-2020/judged': 'msmarco-passage-trec-dl-2020-judged-20230107-training',
    'nfcorpus/test': 'nfcorpus-test-20230107-training',
    'vaswani': 'vaswani-20230107-training',
    'wapo/v2/trec-core-2018': 'wapo-v2-trec-core-2018-20230107-training',
    'longeval/heldout': 'longeval-heldout-20230513-training',
    'longeval/long-september': 'longeval-long-september-20230513-training',
    'longeval/short-july': 'longeval-short-july-20230513-training',
    'longeval/train': 'longeval-train-20230513-training',
    'trec-tip-of-the-tongue/dev': 'trec-tip-of-the-tongue-dev-20230607-training',
}

TIREX_DATASETS = IRDS_TO_TIREX_DATASET.values()



import pandas as pd
from glob import glob
import gzip
import json
import os
from typing import Any, List, Iterable, Dict, Union, Generator
from pathlib import Path
import logging


def parse_jsonl_line(input: Union[str, bytearray, bytes], load_default_text: bool) -> Dict:
    """
    Deseralizes the line using JSON deserialization. Optionally strips the 'original_query' and 'original_document'
    fields from the resulting object and converts the qid and docno fields to strings.

    :param str | bytearray | bytes input: A json-serialized string.
    :param bool load_default_text: If true, the original_query and original_document fields are removed and the qid and
        docno values are converted to strings.
    :return: The deserialized and (optionally) processed object.
    :rtype: dict

    :Example:
        >>> parse_jsonl_line('{}', False)
        {}
        >>> parse_jsonl_line('{"original_query": "xxxx"}', False)
        {'original_query': 'xxxx'}
        >>> parse_jsonl_line('{"original_query": "xxxx"}', True)
        {}
        >>> parse_jsonl_line('{"original_query": "xxxx", "qid": 42, "pi": 3.14}', False)
        {'original_query': 'xxxx', 'qid': 42, 'pi': 3.14}
        >>> parse_jsonl_line('{"original_query": "xxxx", "qid": 42, "pi": 3.14}', True)
        {'qid': '42', 'pi': 3.14}
    """
    obj = json.loads(input)
    assert isinstance(obj, dict)
    if load_default_text:
        for field_to_del in ['original_query', 'original_document']:
            obj.pop(field_to_del, None)

        for field_to_str in ['qid', 'docno']:
            if field_to_str in obj:
                obj[field_to_str] = str(obj[field_to_str])

    return obj


def stream_all_lines(input_file: Union[str, Iterable[bytes]], load_default_text: bool) -> Generator[Dict, Any, Any]:
    """
    .. todo:: add documentation
    .. todo:: this function has two semantics: handling a file and handling file-contents
    """
    if type(input_file) is str:
        if not os.path.isfile(input_file):
            return

        if input_file.endswith('.gz'):
            with gzip.open(input_file, 'rt', encoding='utf-8') as f:
                yield from stream_all_lines(f, load_default_text)
        else:
            with open(input_file, 'r') as f:
                yield from stream_all_lines(f, load_default_text)

        return

    for line in input_file:
        yield parse_jsonl_line(line, load_default_text)


def all_lines_to_pandas(input_file: Union[str, Iterable[str]], load_default_text: bool) -> pd.DataFrame:
    """
    .. todo:: add documentation
    .. todo:: this function has two semantics: handling a file and handling file-contents
    """
    if type(input_file) is str:
        if input_file.endswith('.gz'):
            with gzip.open(input_file, 'rt', encoding='utf-8') as f:
                return all_lines_to_pandas(f, load_default_text)
        else:
            with open(input_file, 'r') as f:
                return all_lines_to_pandas(f, load_default_text)

    import pandas as pd
    ret = []

    for line in input_file:
        ret += [parse_jsonl_line(line, load_default_text)]

    return pd.DataFrame(ret)


def __num(input: str) -> Union[str, int, float]:
    """
    Converts the input to an int or float if possible. Returns the inputted string otherwise.

    :param str input: The string that should be converted to a float or int if possible.
    :return: The intrepteted input.
    :rtype: str | int | float

    :Example:
        >>> __num("hello world")
        "hello world"
        >>> __num("-42")
        -42
        >>> __num("3.5")
        3.5
        >>> __num("2e-6")
        2e-6
        >>> __num(" -42")
        " -42"
    """
    try:
        return int(input)
    except ValueError:
        try:
            return float(input)
        except ValueError:
            return input


def run_cmd(cmd: List[str], ignore_failure=False):
    import subprocess
    exit_code = subprocess.call(cmd)

    if not ignore_failure and exit_code != 0:
        raise ValueError(f'Command {cmd} did exit with return code {exit_code}.')


def parse_prototext_key_values(file_name):
    for i in [i for i in open(file_name, 'r').read().split('measure {')]:
        ret = {}
        for l in i.split('\n'):
            if len(l.split(':')) < 2:
                continue
            elif len(l.split(':')) > 2:
                raise ValueError(f'Could not parse "{l}".')
            key = l.split(':')[0]
            value = l.split(':')[1].split('"')[1]
            ret[key.strip()] = __num(value.strip())
        if len(ret) > 0:
            yield ret


def all_environment_variables_for_github_action_or_fail(params):
    ret = {}

    for i in params:
        if len(i.split('=')) != 2:
            raise ValueError(f"Expect that exactly one '=' occurs in each passed parameter, got: '{i}'")

        key, value = i.split('=')

        if key in ret:
            raise ValueError(f"Got duplicated key: '{key}'")
        
        ret[key] = value

    expected_keys = ['GITHUB_SHA', 'TIRA_VM_ID', 'TIRA_TASK_ID', 'TIRA_DOCKER_REGISTRY_TOKEN', 'TIRA_DOCKER_REGISTRY_USER',
                     'TIRA_CLIENT_TOKEN', 'TIRA_CLIENT_USER', 'TIRA_CODE_REPOSITORY_ID']

    for k in expected_keys:
        if k not in ret or not ret[k]:
            raise ValueError(f'I need the parameter {k} to continue, but could not find one or it is empty. This likely is a configuration error, e.g., due to missing secrets.')

    if 'TIRA_JUPYTER_NOTEBOOK' in ret:
        for to_del in ['TIRA_DOCKER_FILE', 'TIRA_DOCKER_PATH', 'TIRA_COMMAND']:
            if to_del in ret:
                del ret[to_del]
        ret['IMAGE_TAG'] = f'registry.webis.de/code-research/tira/tira-user-{ret["TIRA_VM_ID"]}/submission:{ret["GITHUB_SHA"]}'
    else:
        for expected_key in ['TIRA_DOCKER_FILE', 'TIRA_DOCKER_PATH', 'TIRA_COMMAND']:
            if k not in ret or not ret[k]:
                raise ValueError(f'I need the parameter {k} to continue, but could not find one or it is empty. This likely is a configuration error, e.g., due to missing secrets.')
            
            ret['IMAGE_TAG'] = f'registry.webis.de/code-research/tira/tira-user-{ret["TIRA_VM_ID"]}/submission-{ret["TIRA_DOCKER_PATH"].replace("/", "-").replace(" ", "-")}:{ret["GITHUB_SHA"]}'

    return [k + '=' + v for k, v in ret.items()]


def load_output_of_directory(directory: Path, evaluation: bool=False) -> Union[Dict, pd.DataFrame]:
    files = glob(str(directory) + '/*')

    if evaluation:
        files = [i for i in files if i.endswith('.prototext')]

    if len(files) != 1:
        raise ValueError('Expected exactly one output file. Got: ', files)

    files = files[0]

    logging.debug(f'Read file from {files}')

    if evaluation:
        ret = {}
        for i in [i for i in open(files, 'r').read().split('measure') if 'key:' in i and 'value:' in i]:
            key = i.split('key:')[1].split('value')[0].split('"')[1]
            value = i.split('key:')[1].split('value')[1].split('"')[1]

            ret[key.strip()] = __num(value.strip())

        return ret
    else:
        return pd.read_json(files, lines=True, orient='records')



import requests
import json
import pandas as pd
from pathlib import Path
import os
import zipfile
import io
import time
from random import randint
from tira.pyterrier_integration import PyTerrierIntegration, PyTerrierAnceIntegration
from tira.pandas_integration import PandasIntegration
from tira.local_execution_integration import LocalExecutionIntegration
import logging
from .tira_client import TiraClient
from typing import Optional, List, Dict, Union
from functools import lru_cache
from tira.tira_redirects import redirects, mirror_url, dataset_ir_redirects, RESOURCE_REDIRECTS
from tqdm import tqdm
import hashlib

class Client(TiraClient):
    base_url: str

    def __init__(self, base_url: Optional[str]=None, api_key: str=None, failsave_retries: int=5, failsave_max_delay: int=15, api_user_name: Optional[str]=None):
        self.base_url = base_url or 'https://www.tira.io'
        self.tira_cache_dir = os.environ.get('TIRA_CACHE_DIR', os.path.expanduser('~') + '/.tira')
        self.json_cache = {}

        if api_key is None:
            self.api_key = self.load_settings()['api_key']
            self.api_user_name = self.load_settings()['api_user_name']
        else:
            self.api_key = api_key
            self.api_user_name = api_user_name

        self.failsave_retries = 1
        if self.api_key != 'no-api-key':
            self.fail_if_api_key_is_invalid()
        self.pd = PandasIntegration(self)
        self.pt = PyTerrierIntegration(self)
        self.pt_ance = PyTerrierAnceIntegration(self)
        self.local_execution = LocalExecutionIntegration(self)

        self.failsave_retries = failsave_retries
        self.failsave_max_delay = failsave_max_delay

    def load_settings(self):
        try:
            return json.load(open(self.tira_cache_dir + '/.tira-settings.json', 'r'))
        except Exception:
            logging.info(f'No settings given in {self.tira_cache_dir}/.tira-settings.json. I will use defaults.')
            return {'api_key': 'no-api-key', 'api_user_name': 'no-api-key-user'}

    def update_settings(self, k, v):
        settings = self.load_settings()
        settings[k] = v
        json.dump(settings, open(Path(self.tira_cache_dir) / '.tira-settings.json', 'w+'))

        if k == 'api_key':
            self.api_key = settings['api_key']

    def api_key_is_valid(self):
        role = self.json_response('/api/role')

        if (self.api_user_name is None or self.api_user_name == 'no-api-key-user') and role and 'context' in role and 'user_id' in role['context'] and role['context']['user_id']:
            self.api_user_name = role['context']['user_id']

        return role and 'status' in role and 'role' in role and 0 == role['status'] and 'user_id' in role['context'] and role['context']['user_id'] and 'role' in role['context'] and role['context']['role'] and 'guest' != role['context']['role']

    def fail_if_api_key_is_invalid(self):
        if not self.api_key_is_valid():
            role = self.json_response('/api/role')
            raise ValueError('It seems like the api key is invalid. Got: ', role)

    def datasets(self, task):
        return json.loads(self.json_response(f'/api/datasets_by_task/{task}')['context']['datasets'])

    def docker_software_id(self, approach):
        return self.docker_software(approach)['docker_software_id']

    def all_softwares(self, task_id):
        """
        Return all public submissions.
        """
        return [task_id + '/' + i['vm_id'] + '/' + i['display_name'] for i in self.json_response(f'/api/task/{task_id}/public-submissions')['context']['public_submissions']]

    def docker_software(self, approach):
        task, team, software = approach.split('/')
        return self.json_response(f'/api/task/{task}/submission-details/{team}/{software}')['context']['submission']

    def docker_credentials(self, task_name, team_name):
        ret = self.metadata_for_task(task_name, team_name)
        if ret and 'status' in ret and ret['status'] == 0 and 'context' in ret and 'docker' in ret['context']:
            return ret['context']['docker']['docker_registry_user'], ret['context']['docker']['docker_registry_token'], self.docker_registry()
        return None, None, self.docker_registry()

    def docker_software_details(self, approach):
        task, team, software = approach.split('/')
        ret = self.json_response(f'/task/{task}/vm/{team}/software_details/{software}')

        return ret

    def metadata_for_task(self, task_name, team_name=None):
        if team_name is None:
            return self.json_response(f'/api/task/{task_name}')
        else:
            return self.json_response(f'/api/task/{task_name}/user/{team_name}')

    def add_docker_software(self, image, command, tira_vm_id, tira_task_id, code_repository_id, build_environment, previous_stages=[]):
        headers = {
            'Api-Key': self.api_key,
            'Api-Username': self.api_user_name,
            'Accept': 'application/json',
            'Content-Type': 'application/json',
        }
        self.fail_if_api_key_is_invalid()
        url = f'{self.base_url}/task/{tira_task_id}/vm/{tira_vm_id}/add_software/docker'
        content = {"action": "post", "image": image, "command": command, "code_repository_id": code_repository_id,"build_environment": json.dumps(build_environment)}

        if previous_stages and len(previous_stages) > 0:
            content['inputJob'] = previous_stages

        ret = requests.post(url, headers=headers, json=content)
        ret = ret.content.decode('utf8')
        ret = json.loads(ret)
        assert ret['status'] == 0
        
        print(f'Software with name {ret["context"]["display_name"]} was created.')
        logging.info(f'Software with name {ret["context"]["display_name"]} was created.')
        logging.info(f'Please visit {self.base_url}/submit/{tira_task_id}/user/{tira_vm_id}/docker-submission to run your software.')

    def submissions(self, task, dataset):
        response = self.json_response(f'/api/submissions/{task}/{dataset}')['context']
        ret = []
        
        for vm in response['vms']:
            for run in vm['runs']:
                if 'review' in run:
                    for k,v in run['review'].items():
                        run['review_' + k] = v
                    del run['review']

                ret += [{**{'task': response['task_id'], 'dataset': response['dataset_id'], 'team': vm['vm_id']}, **run}]

        return pd.DataFrame(ret)

    def upload_submissions(self, task_id, vm_id, upload_id, dataset=None):
        ret = self.json_response(f'/api/upload-group-details/{task_id}/{vm_id}/{upload_id}')
        ret = ret['context']['upload_group_details']['runs']
        
        return [i for i in ret if dataset is None or dataset == i['dataset']]

    def submissions_with_evaluation_or_none(self, task, dataset, team, software):
        """ This method returns all runs of the specified software in the task on the dataset by the team.
        This is especially suitable to batch evaluate all submissions of the software because the evaluation is none if no successfull evaluation was conducted (or there is a new evaluator).
        E.g., by code like:

        .. code:: py

            for approach in ['approach-1', ..., 'approach-n]:
                runs_for_approach = tira.submissions_with_evaluation_or_none(task, dataset, team, approach)
                for i in runs_for_approach:
                    if not i['evaluation']:
                        tira.evaluate_run(team, dataset, i['run_id'])
        """
        submissions = self.submissions(task, dataset)
        evaluations = self.evaluations(task, dataset, join_submissions=False)
        run_to_evaluation = {}
        for _, i in evaluations.iterrows():
            i = i.to_dict()
            run_id = i["run_id"]
            del i['task']
            del i['dataset']
            del i['team']
            del i['run_id']
            run_to_evaluation[run_id] = i

        if len(submissions) < 1:
            return []

        submissions = submissions[(submissions['task'] == task) & (submissions['dataset'] == dataset) & (submissions['team'] == team) & (submissions['software'] == software)]
        ret = []
        for run_id in submissions.run_id.unique():
            ret += [{"run_id": run_id, "task": task, "dataset": dataset, "team": team, "software": software, "evaluation": run_to_evaluation.get(run_id, None)}]

        return ret

    def evaluations(self, task, dataset, join_submissions=True):
        response = self.json_response(f'/api/evaluations/{task}/{dataset}')['context']
        ret = []
        evaluation_keys = response['ev_keys']

        if join_submissions:
            runs_to_join = {}
            for _, i in self.submissions(task, dataset).iterrows():
                i = i.to_dict()
                runs_to_join[(i['team'], i['run_id'])] = {'software': i['software'], 'input_run_id': i['input_run_id'], 'is_upload': i['is_upload'], 'is_docker': i['is_docker']}

        for evaluation in response['evaluations']:
            run = {'task': response['task_id'], 'dataset': response['dataset_id'], 'team': evaluation['vm_id'], 'run_id': evaluation['input_run_id'], 'evaluation_run_id': evaluation['run_id'], 'published': evaluation['published'], 'blinded': evaluation['blinded']}

            if join_submissions and (run['team'], run['run_id']) in runs_to_join:
                software = runs_to_join[(run['team'], run['run_id'])]
                for k, v in software.items():
                    run[k] = v

            for measure_id, measure in zip(range(len(evaluation_keys)), evaluation_keys):
                run[measure] = evaluation['measures'][measure_id]

            ret += [run]

        return pd.DataFrame(ret)

    def run_was_already_executed_on_dataset(self, approach, dataset):
        return self.get_run_execution_or_none(approach, dataset) is not None

    def load_resource(self, resource:str):
        """Load a resource (usually a zip) from TIRA/Zenodo. Serves as utikity function in case some additional resources must be loaded.

        Args:
            resource (str): The resource identifier

        Raises:
            ValueError: If the resource is not known.

        Returns:
            str: The path to the downloaded resource.
        """
        target_file = f'{self.tira_cache_dir}/raw_resources/{resource}'
        if os.path.exists(target_file):
            return target_file

        if resource not in RESOURCE_REDIRECTS:
            raise ValueError(f'Resource {resource} not supported.')

        os.makedirs(f'{self.tira_cache_dir}/raw_resources', exist_ok=True)
        self.download_and_extract_zip(RESOURCE_REDIRECTS[resource], target_file, extract=False)
        return target_file

    def get_run_output(self, approach, dataset, allow_without_evaluation=False):
        """
        Downloads the run (or uses the cached version) of the specified approach on the specified dataset.
        Returns the directory containing the outputs of the run.
        """
        mounted_output_in_sandbox = self.input_run_in_sandbox(approach)
        if mounted_output_in_sandbox:
            return mounted_output_in_sandbox

        task, team, software = approach.split('/')
        if '/' in dataset:
            dataset = dataset.split('/')[-1]
        #raise ValueError('Add unit test for the above')
        run_execution = self.get_run_execution_or_none(approach, dataset)

        if run_execution:
            return self.download_zip_to_cache_directory(task, dataset, team, run_execution['run_id'])

        run_execution = self.submissions_with_evaluation_or_none(task, dataset, team, software)
        run_execution = [i for i in run_execution if ('evaluation' in i and i['evaluation']) or allow_without_evaluation]

        if run_execution is None or len(run_execution) < 1:
            raise ValueError(f'Could not get run for approach "{approach}" on dataset "{dataset}".')

        return self.download_zip_to_cache_directory(run_execution[0]['task'], run_execution[0]['dataset'], run_execution[0]['team'], run_execution[0]['run_id'])

    def public_runs(self, task, dataset, team, software):
        ret = self.json_response(f'/api/list-runs/{task}/{dataset}/{team}/' + software.replace(' ', '%20'))
        if ret and 'context' in ret and 'runs' in ret['context'] and ret['context']['runs']:
            return ret['context']
        else:
            return None

    def get_run_execution_or_none(self, approach, dataset, previous_stage_run_id=None):
        task, team, software = approach.split('/')
        redirect = redirects(approach, dataset)

        if redirect is not None and 'run_id' in redirect and redirect['run_id'] is not None:
            return {'task': task, 'dataset': dataset, 'team': team, 'run_id': redirect['run_id']}

        public_runs = self.public_runs(task, dataset, team, software)
        if public_runs:
            return {'task': task, 'dataset': dataset, 'team': team, 'run_id': public_runs['runs'][0]}

        df_eval = self.submissions_of_team(task=task, dataset=dataset, team=team)
        if len(df_eval) <= 0:
            return None

        ret = df_eval[(df_eval['dataset'] == dataset) & (df_eval['software'] == software)]
        if len(ret) <= 0:
            return None
        
        if team:
            ret = ret[ret['team'] == team]

        # FIXME: Is this really necessary or is it checked with the if len(ret) <= 0 later on?
        if len(ret) <= 0:
            return None

        if previous_stage_run_id:
            ret = ret[ret['input_run_id'] == previous_stage_run_id]

        if len(ret) <= 0:
            return None

        return ret[['task', 'dataset', 'team', 'run_id']].iloc[0].to_dict()
        
    def download_run(self, task, dataset, software, team=None, previous_stage=None, return_metadata=False):
        if '/' in dataset:
            dataset = dataset.split('/')[-1]
        ret = self.get_run_execution_or_none(f'{task}/{team}/{software}', dataset, previous_stage)
        if not ret:
            raise ValueError(f'I could not find a run for the filter criteria task="{task}", dataset="{dataset}", software="{software}", team={team}, previous_stage={previous_stage}')
        run_id = ret['run_id']
        
        ret = self.download_zip_to_cache_directory(**ret)
        ret = pd.read_csv(ret + '/run.txt', sep='\\s+', names=["query", "q0", "docid", "rank", "score", "system"], dtype={"query": str, "docid": str})
        if return_metadata:
            return ret, run_id
        else:
            return ret

    def download_evaluation(self, task, dataset, software, team):
        ret = self.submissions_with_evaluation_or_none(task, dataset, team, software)
        if not ret or len(ret) < 1:
            raise ValueError(f'I could not find a run for the filter criteria task="{task}", dataset="{dataset}", software="{software}", team={team}.')
        run_id = ret[0]['run_id']

        submissions = self.submissions(task, dataset)
        submissions = submissions[(submissions['input_run_id'] == run_id) & (submissions['is_evaluation'])]

        if submissions is None or len(submissions) < 1:
            raise ValueError(f'I could not find a evaluation for the filter criteria task="{task}", dataset="{dataset}", software="{software}", team={team}, run_id={run_id}.')

        return self.download_zip_to_cache_directory(task, dataset, team, submissions.iloc[0].to_dict()['run_id'])

    def download_dataset(self, task, dataset, truth_dataset=False):
        """
        Download the dataset. Set truth_dataset to true to load the truth used for evaluations.
        """
        if 'TIRA_INPUT_DATASET' in os.environ:
            return os.environ['TIRA_INPUT_DATASET']

        dataset = dataset_ir_redirects(dataset)

        target_dir = f'{self.tira_cache_dir}/extracted_datasets/{task}/{dataset}/'
        suffix = ("input-data" if not truth_dataset else "truth-data")
        if os.path.isdir(target_dir + suffix):
            return target_dir + suffix
        data_type = 'training' if dataset.endswith('-training') else 'test'
        self.download_and_extract_zip(f'{self.base_url}/data-download/{data_type}/input-{("" if not truth_dataset else "truth")}/{dataset}.zip', target_dir)

        os.rename(target_dir + f'/{dataset}', target_dir + suffix)

        return target_dir + suffix

    def download_zip_to_cache_directory(self, task, dataset, team, run_id):
        target_dir = f'{self.tira_cache_dir}/extracted_runs/{task}/{dataset}/{team}'

        if os.path.isdir(target_dir + f'/{run_id}'):
            return target_dir + f'/{run_id}/output'

        self.download_and_extract_zip(f'{self.base_url}/task/{task}/user/{team}/dataset/{dataset}/download/{run_id}.zip', target_dir)

        return target_dir + f'/{run_id}/output'

    def add_run_to_leaderboard(self, task, team, dataset, evaluation_run_id=None, run_id=None):
        """
        Publish the specified run to the leaderboard.
        
        This is especially suitable to batch add all submissions of submissions, e.g., by code like:

        .. code:: py

            for approach in ['approach-1', ..., 'approach-n']:
                runs_for_approach = tira.submissions_with_evaluation_or_none(task, dataset, team, approach)
                for i in runs_for_approach:
                    if i['evaluation']:
                        tira.add_run_to_leaderboard(TASK, team, dataset, run_id=i['run_id'])
        
        """
        if run_id and evaluation_run_id:
            raise ValueError(f'Please pass either a evaluation_run_id or a run_id, but both were passed: evaluation_run_id={evaluation_run_id}, run_id={run_id}')
        if run_id and not evaluation_run_id:
            submissions = self.submissions(task, dataset)
            submissions = submissions[(submissions['input_run_id'] == run_id) & (submissions['is_evaluation'])]

            for evaluation_run_id in submissions['run_id'].unique():
                self.add_run_to_leaderboard(task, team, dataset, evaluation_run_id=evaluation_run_id)

        if evaluation_run_id:
            logging.info(f'Publish run: {evaluation_run_id}.')
            ret = self.json_response(f'/publish/{team}/{dataset}/{evaluation_run_id}/true')

            if ('status' not in ret) or ('0' != ret['status']) or ('published' not in ret) or (not ret['published']):
                raise ValueError(f'Adding the run to the leaderboard failed. Got {ret}')

    def get_configuration_of_evaluation(self, task_id, dataset_id):
        """ Get the configuration of the evaluator for the passed dataset inside the task specified by task_id.
        """
        ret = self.json_response(f'/api/configuration-of-evaluation/{task_id}/{dataset_id}')

        if 'status' not in ret or '0' != str(ret['status']):
            raise ValueError(f'Failed to load configuration of an evaluator. Got {ret}')

        return ret['context']['dataset']

    def evaluate_run(self, team, dataset, run_id):
        """ Evaluate the run of the specified team and identified by the run_id (the run must be submitted on the specified dataset).
        """
        ret = self.json_response(f'/grpc/{team}/run_eval/{dataset}/{run_id}')

        if 'status' not in ret or '0' != str(ret['status']):
            raise ValueError(f'Failed to evaluate the run. Got {ret}')

        return ret

    def download_and_extract_zip(self, url, target_dir, extract=True):
        url = redirects(url=url)['urls'][0]
        if url.split('://')[1].startswith('files.webis.de'):
            print(f'Download from the Incubator: {url}')
            print('\tThis is only used for last spot checks before archival to Zenodo.')
        
        if  url.split('://')[1].startswith('zenodo.org'):
            print(f'Download from Zenodo: {url}')


        for _ in range(self.failsave_retries):
            status_code = None
            try:
                headers={"Api-Key": self.api_key, "Api-Username": self.api_user_name}
                if self.api_key == 'no-api-key':
                    del headers["Api-Key"]
                if self.api_user_name == 'no-api-key-user':
                    del headers["Api-Username"]

                r = requests.get(url, headers=headers, stream=True)
                total = int(r.headers.get('content-length', 0))
                status_code = r.status_code
                if status_code < 200 or status_code >= 300:
                    raise ValueError(f'Got non 200 status code {status_code} for {url}.')

                response_content = io.BytesIO()
                with tqdm(desc='Download', total=total, unit='iB', unit_scale=True, unit_divisor=1024,) as bar:
                    for data in r.iter_content(chunk_size=1024):
                        size = response_content.write(data)
                        bar.update(size)
                if extract:
                    print('Download finished. Extract...')
                    z = zipfile.ZipFile(response_content)
                    z.extractall(target_dir)
                    print('Extraction finished: ', target_dir)
                else:
                    print('Download finished. Persist...')
                    with open(target_dir, 'wb') as file_out:
                        file_out.write(response_content.getbuffer())
                    print('Download finished: ', target_dir)

                return
            except Exception as e:
                sleep_time = randint(1, self.failsave_max_delay)
                print(f'Code: {status_code}')
                print(f'Error occured while fetching {url}. I will sleep {sleep_time} seconds and continue.')
                url = mirror_url(url)
                time.sleep(sleep_time)

    def login(self, token):
        self.api_key = token

        if not self.api_key_is_valid():
            print('The api key {token} is not valid')
            raise ValueError(f'The api key {token} is invalid.')

        self.update_settings('api_key', token)

    def get_authentication_cookie(self, user, password):
        resp = requests.get(f'{self.base_url}/session/csrf', headers={'x-requested-with': 'XMLHttpRequest'})

        header = {
            'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
            'cookie': '_forum_session=' + resp.cookies['_forum_session'],
            'x-csrf-token': resp.json()['csrf'],
            'x-requested-with': 'XMLHttpRequest'
        }

        resp = requests.post(f'{self.base_url}/session', data=f'login={user}&password={password}', headers=header)

        return f'_t={resp.cookies["_t"]}; _forum_session={resp.cookies["_forum_session"]}'

    def run_software(self, approach, dataset, resources, rerank_dataset='none'):
        task, team, software = approach.split('/')
        authentication_cookie = self.get_authentication_cookie(self.load_settings()['user'], self.load_settings()['password'])

        software_id = self.docker_software_id(approach)
        if not software_id:
            raise ValueError(f'Could not find software id for "{approach}". Got: "{software_id}".')

        url = f'{self.base_url}/grpc/{task}/{team}/run_execute/docker/{dataset}/{software_id}/{resources}/{rerank_dataset}'
        logging.info(f'Start software...\n\t{url}\n')

        csrf_token = self.get_csrf_token()
        headers = {
            # 'Api-Key': self.api_key,
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'Cookie': authentication_cookie,
            'x-csrftoken': csrf_token,
        }

        ret = requests.post(url, headers=headers, json={"csrfmiddlewaretoken": csrf_token, "action": "post"})
        ret = ret.content.decode('utf8')
        logging.info(ret)
        ret = json.loads(ret)
        assert ret['status'] == 0
    
    def get_upload_group_id(self, task_id: str, vm_id: str, display_name: str) -> int:
        """Get the id of the upload group of user specified with vm_id for the task task_id with the display_name. Raises an error if no matching upload_group was found."""
        url = f"/api/submissions-for-task/{task_id}/{vm_id}/upload"
        ret = self.json_response(url)
        if 'context' not in ret or 'all_uploadgroups' not in ret['context']:
            logging.error(f"Failed to get upload id, response does not contain the expected fields.")
            logging.debug(response.content)
            raise ValueError(f'Invalid response for request {url}: {ret}.')

        for upload_group in ret['context']['all_uploadgroups']:
            if display_name is not None and upload_group['display_name'] == display_name:
                return upload_group['id']

        logging.error(f"Could not find upload with display_name {display_name} for task {task_id} of user {vm_id}. Got:", ret['context']['all_uploadgroups'])
        raise ValueError("Could not find upload with display_name {display_name} for task {task_id} of user {vm_id}. Got:", ret['context']['all_uploadgroups'])
    
    def create_upload_group(self, task_id: str, vm_id: str, display_name: str) -> Optional[str]:
        # TODO: check that task_id and vm_id don't contain illegal characters (e.g., '/')
        # TODO: Make this idempotent: reuse existing upload group if it already exists.
        url = f"{self.base_url}/task/{task_id}/vm/{vm_id}/add_software/upload"
        logging.debug(f"Creating a new upload at {url}")
        ret = json_response(url)
            
        logging.debug(f"Created new upload with id {content.upload}")
        return content.upload

    def upload_run(self, file_path: Path, dataset_id: str, approach: str=None, task_id: str=None, vm_id: str=None, upload_id: str=None, allow_multiple_uploads=False) -> bool:
        """
        Returns true if the upload was successfull, false if not, or none if it was already uploaded.
        """
        logging.info(f"Submitting {upload_id} for Task {task_id}:{dataset_id} on VM {vm_id}")
        if approach:
            task_id, vm_id, display_name = approach.split('/')
            upload_id = self.get_upload_group_id(task_id, vm_id, display_name)    

        previous_uploads = self.upload_submissions(task_id, vm_id, upload_id, dataset_id)
        if len(previous_uploads) > 0:
            logging.warn(f'Skip upload of file {file_path} for dataset {dataset_id} because there are already {len(previous_uploads)} for this dataset. Pass allow_multiple_uploads=True to upload a new dataset or delete the uploads in the UI.')
            return None

        self.fail_if_api_key_is_invalid()

        # TODO: check that task_id and vm_id don't contain illegal characters (e.g., '/')
        url = f"/task/{task_id}/vm/{vm_id}/upload/{dataset_id}/{upload_id}"
        logging.info(f"Submitting the runfile at {url}")
        response = self.execute_post_return_json(url, file_path=file_path)

        return 'status' in response and response['status'] == 0 and 'message' in response and response['message'] == 'ok' and 'new_run' in response

    def get_csrf_token(self):
        ret = requests.get(f'{self.base_url}/', headers={"Api-Key": self.api_key})
        return ret.content.decode('utf-8').split('name="csrfmiddlewaretoken" value="')[1].split('"')[0]

    def execute_post_return_json(self, endpoint: str, params: Optional[Union[Dict, List[tuple], bytes]] = None, file_path: Path=None) -> Dict: 
        assert endpoint.startswith('/')
        headers = {
            'Api-Key': self.api_key,
            'Api-Username': self.api_user_name,
            'Accept': 'application/json',
        }
        for _ in range(self.failsave_retries):
            try:
                files={'file': open(file_path, 'rb')}

                resp = requests.post(url=f'{self.base_url}{endpoint}', files=files, headers=headers, params=params)
                if resp.status_code not in {200, 202}:
                    raise ValueError(f'Got statuscode {resp.status_code} for {endpoint}. Got {resp.content}')
                else:
                    break
            except Exception as e:
                sleep_time = randint(1, self.failsave_max_delay)
                logging.warn(f'Error occured while fetching {endpoint}. Code: {resp.status_code}. I will sleep {sleep_time} seconds and continue.', exc_info=e)
                time.sleep(sleep_time)

        return resp.json()

    @lru_cache(maxsize=None)
    def json_response(self, endpoint: str, params: Optional[Union[Dict, List[tuple], bytes]] = None):
        assert endpoint.startswith('/')
        headers = {"Accept": "application/json"}

        if self.api_key != 'no-api-key':
            headers["Api-Key"] = self.api_key
        if self.api_user_name != 'no-api-key-user':
            headers["Api-Username"] = self.api_user_name

        for _ in range(self.failsave_retries):
            try:
                resp = requests.get(url=f'{self.base_url}{endpoint}', headers=headers, params=params)
                if resp.status_code not in {200, 202}:
                    raise ValueError(f'Got statuscode {resp.status_code} for {endpoint}. Got {resp}')
                else:
                    break
            except Exception as e:
                sleep_time = randint(1, self.failsave_max_delay)
                logging.warn(f'Error occured while fetching {endpoint}. Code: {resp.status_code}. I will sleep {sleep_time} seconds and continue.', exc_info=e)
                time.sleep(sleep_time)

        return resp.json()


    def __listdir_failsave(self, path: str):
        try:
            return os.listdir(path)
        except:
            return []


    def input_run_in_sandbox(self, approach:str):
        """
        Returns the directory with the outputs of an approach in mounted into the sandbox. returns None if not in the sandbox.
        """
        if 'inputRun' not in os.environ:
            return None

        input_run = os.environ['inputRun']
        input_run_mapping_file = Path(self.tira_cache_dir) / (hashlib.md5(input_run.encode('utf-8')).hexdigest()[:6] +'-input-run-mapping.json')

        files_in_input_dir = self.__listdir_failsave(input_run)
        if '1' not in files_in_input_dir or '2' not in files_in_input_dir:
            return input_run

        input_run_mapping = {}
        if input_run_mapping_file.exists():
            input_run_mapping = json.load(open(input_run_mapping_file, 'r'))

        if approach not in input_run_mapping:
            next_id = max(list(input_run_mapping.values()) + [0]) + 1
            input_run_mapping[approach] = next_id
            os.makedirs(self.tira_cache_dir, exist_ok=True)
            json.dump(input_run_mapping, open(input_run_mapping_file, 'w+'))

        return input_run + '/' + str(input_run_mapping[approach])




from tira.rest_api_client import Client
from tira.third_party_integrations import ensure_pyterrier_is_loaded
import os
import unittest

def get_input_run_mounts(input_run, approach_identifier):
    os.environ['inputRun'] = input_run
    actual_dir = Client().input_run_in_sandbox(approach_identifier)
    del os.environ['inputRun']
    actual_files = []
    try:
        actual_files = set(os.listdir(actual_dir))
    except:
        pass

    return actual_dir, actual_files

def tira_cli_download_run_output(input_run, approach_identifier):
    os.environ['inputRun'] = input_run
    actual_dir = Client().get_run_output(approach_identifier, 'dataset')
    del os.environ['inputRun']
    actual_files = []
    try:
        actual_files = set(os.listdir(actual_dir))
    except:
        pass

    return actual_dir, actual_files

def tira_cli_download_dataset(dataset_path):
    os.environ['TIRA_INPUT_DATASET'] = dataset_path
    actual_dir = Client().download_dataset(None, 'dataset')
    del os.environ['TIRA_INPUT_DATASET']
    actual_files = []
    try:
        actual_files = set(os.listdir(actual_dir))
    except:
        pass

    return actual_dir, actual_files

def get_pt_index_in_sandbox(input_run, approach_identifier):
    ensure_pyterrier_is_loaded()
    os.environ['inputRun'] = input_run

    try:
        ret = Client().pt.index(approach_identifier, 'dataset_id')
        del os.environ['inputRun']
        return ret
    except:
        del os.environ['inputRun']
        return None

class InputRunMountTest(unittest.TestCase):
    def test_for_single_input_run_with_four_files(self):
        expected_dir = 'tests/resources/sample-input-full-rank'
        expected_files = set(['documents.jsonl', 'metadata.json', 'queries.jsonl', 'queries.xml'])

        actual_dir, actual_files = get_input_run_mounts('tests/resources/sample-input-full-rank', 'approach-identifier')
        
        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)

    def test_for_single_input_run_with_single_file(self):
        expected_dir = 'tests/resources/re-ranking-outputs/'
        expected_files = set(['rerank.jsonl.gz'])

        actual_dir, actual_files = get_input_run_mounts('tests/resources/re-ranking-outputs/', 'some-identifier')
        
        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)

    def test_for_multi_input_runs_01(self):
        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-01/', 'some-identifier')
        self.assertEqual('tests/resources/input-run-01//1', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-01/', 'some-identifier')
        self.assertEqual('tests/resources/input-run-01//1', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-01/', 'other-approach-identifier')
        self.assertEqual('tests/resources/input-run-01//2', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-01/', 'some-identifier')
        self.assertEqual('tests/resources/input-run-01//1', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-01/', 'other-approach-identifier')
        self.assertEqual('tests/resources/input-run-01//2', actual_dir)

    def test_for_multi_input_runs_02(self):
        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'a')
        self.assertEqual('tests/resources/input-run-02//1', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'b')
        self.assertEqual('tests/resources/input-run-02//2', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'c')
        self.assertEqual('tests/resources/input-run-02//3', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'a')
        self.assertEqual('tests/resources/input-run-02//1', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'b')
        self.assertEqual('tests/resources/input-run-02//2', actual_dir)

        actual_dir, _ = get_input_run_mounts('tests/resources/input-run-02/', 'c')
        self.assertEqual('tests/resources/input-run-02//3', actual_dir)

    def test_for_index_that_does_not_exist(self):
        index = get_pt_index_in_sandbox('tests/resources/input-run-02/', 'a')
        self.assertIsNone(index)

    def test_for_index_that_does_exist_with_multi_inputs(self):
        index = get_pt_index_in_sandbox('tests/resources/input-run-02/', 'a')
        self.assertIsNone(index)

        index = get_pt_index_in_sandbox('tests/resources/input-run-02/', 'b')
        self.assertIsNotNone(index)

    def test_for_index_that_does_exist_with_single_input(self):
        index = get_pt_index_in_sandbox('tests/resources/input-run-02/2', 'a')
        self.assertIsNotNone(index)

    def test_for_index_that_does_not_exist_with_single_input(self):
        index = get_pt_index_in_sandbox('tests/resources/input-run-02/1', 'a')
        self.assertIsNone(index)

    def test_cli_download_run_output_01(self):
        expected_dir = 'tests/resources/re-ranking-outputs/'
        expected_files = set(['rerank.jsonl.gz'])
    
        actual_dir, actual_files = tira_cli_download_run_output('tests/resources/re-ranking-outputs/', 'a')

        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)

    def test_cli_download_run_output_02(self):
        expected_dir = 'tests/resources/input-run-02//2'
        expected_files = set(['.gitkeep', 'index'])
    
        tira_cli_download_run_output('tests/resources/input-run-02/', 'a')
        actual_dir, actual_files = tira_cli_download_run_output('tests/resources/input-run-02/', 'b')

        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)

    def test_cli_download_dataset_01(self):
        expected_dir = 'tests/resources/re-ranking-outputs/'
        expected_files = set(['rerank.jsonl.gz'])
    
        actual_dir, actual_files = tira_cli_download_dataset('tests/resources/re-ranking-outputs/')

        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)

    def test_cli_download_dataset_02(self):
        expected_dir = 'tests/resources/input-run-02//2'
        expected_files = set(['index', '.gitkeep'])
    
        actual_dir, actual_files = tira_cli_download_dataset('tests/resources/input-run-02//2')

        self.assertEqual(expected_dir, actual_dir)
        self.assertEqual(expected_files, actual_files)
